@@ -31,7 +31,7 @@
  *  Created by Mengyao Zhao on 6/22/10.
  *  Copyright 2010 Boston College. All rights reserved.
  *	Version 0.1.4
- *	Last revision by Mengyao Zhao on 02/11/16.
+ *	Last revision by Mengyao Zhao on 07/19/16.
  *
  */
 
@@ -529,6 +529,37 @@ end:
 	return bests;
 }
 
+/*!     @function               Produce CIGAR 32-bit unsigned integer from CIGAR operation and CIGAR length
+        @param  length          length of CIGAR
+        @param  op_letter       CIGAR operation character ('M', 'I', etc)
+        @return                 32-bit unsigned integer, representing encoded CIGAR operation and length
+*/
+uint32_t to_cigar_int (uint32_t length, char op_letter)
+{
+        switch (op_letter) {
+                case 'M': /* alignment match (can be a sequence match or mismatch */
+                default:
+                        return length << BAM_CIGAR_SHIFT;
+                case 'S': /* soft clipping (clipped sequences present in SEQ) */
+                        return (length << BAM_CIGAR_SHIFT) | (4u);
+                case 'D': /* deletion from the reference */
+                        return (length << BAM_CIGAR_SHIFT) | (2u);
+                case 'I': /* insertion to the reference */
+                        return (length << BAM_CIGAR_SHIFT) | (1u);
+                case 'H': /* hard clipping (clipped sequences NOT present in SEQ) */
+                        return (length << BAM_CIGAR_SHIFT) | (5u);
+                case 'N': /* skipped region from the reference */
+                        return (length << BAM_CIGAR_SHIFT) | (3u);
+                case 'P': /* padding (silent deletion from padded reference) */
+                        return (length << BAM_CIGAR_SHIFT) | (6u);
+                case '=': /* sequence match */
+                        return (length << BAM_CIGAR_SHIFT) | (7u);
+                case 'X': /* sequence mismatch */
+                        return (length << BAM_CIGAR_SHIFT) | (8u);
+        }
+        return (uint32_t)-1; // This never happens
+}
+
 static cigar* banded_sw (const int8_t* ref,
 				 const int8_t* read,
 				 int32_t refLen,
@@ -855,13 +886,98 @@ void align_destroy (s_align* a) {
 	free(a->cigar);
 	free(a);
 }
-/*
-inline char cigar_int_to_op(uint32_t cigar_int) {
-	return UNLIKELY((cigar_int & 0xfU) > 8) ? 'M': MAPSTR[cigar_int & 0xfU];
+
+uint32_t* add_cigar (uint32_t* new_cigar, int32_t* p, int32_t* s, uint32_t length, char op) {
+	if ((*p) >= (*s)) {
+		++(*s);
+		kroundup32(*s);
+		new_cigar = (uint32_t*)realloc(new_cigar, (*s)*sizeof(uint32_t));
+	}
+	new_cigar[(*p) ++] = to_cigar_int(length, op);
+	return new_cigar;
 }
 
+uint32_t* store_previous_m (int8_t choice,	// 0: current not M, 1: current match, 2: current mismatch
+					   uint32_t* length_m,
+					   uint32_t* length_x,
+					   int32_t* p,
+					   int32_t* s,
+					   uint32_t* new_cigar) {
+
+	if ((*length_m) && (choice == 2 || !choice)) {
+		new_cigar = add_cigar (new_cigar, p, s, (*length_m), '='); 
+		(*length_m) = 0;
+	} else if ((*length_x) && (choice == 1 || !choice)) { 
+		new_cigar = add_cigar (new_cigar, p, s, (*length_x), 'X'); 
+		(*length_x) = 0;
+	}
+	return new_cigar;
+}				
+
+/*! @function:
+     1. Calculate the number of mismatches.
+     2. Modify the cigar string:
+         differentiate matches (=) and mismatches(X); add softclip(S) at the beginning and ending of the original cigar.
+    @return:
+     The number of mismatches.
+	 The cigar and cigarLen are modified.
+*/
+int32_t mark_mismatch (int32_t ref_begin1,
+					   int32_t read_begin1,
+					   int32_t read_end1,
+					   const char* ref,
+					   const char* read,
+					   int32_t readLen,
+					   uint32_t** cigar,
+					   int32_t* cigarLen) {
+
+	int32_t mismatch_length = 0, p = 0, i, length, j, s = *cigarLen + 2;
+	uint32_t *new_cigar = (uint32_t*)malloc(s*sizeof(uint32_t)), length_m = 0,  length_x = 0;
+	char op;
+
+	ref += ref_begin1;
+	read += read_begin1;
+	if (read_begin1 > 0) new_cigar[p ++] = to_cigar_int(read_begin1, 'S');
+	for (i = 0; i < (*cigarLen); ++i) {
+		op = cigar_int_to_op((*cigar)[i]);
+		length = cigar_int_to_len((*cigar)[i]);
+		if (op == 'M') {
+			for (j = 0; j < length; ++j) {
+				fprintf(stderr, "ref[%d]: %c\tread[%d]: %c\n", j, *ref, j, *read);
+				if (*ref != *read) {
+					++ mismatch_length;
+					fprintf(stderr, "length_m: %d\n", length_m);
+					// the previous is match; however the current one is mismatche
+					new_cigar = store_previous_m (2, &length_m, &length_x, &p, &s, new_cigar);			
+					++ length_x;
+				} else {
+					// the previous is mismatch; however the current one is matche
+					new_cigar = store_previous_m (1, &length_m, &length_x, &p, &s, new_cigar);			
+					++ length_m;
+				}
+				++ ref;
+				++ read;
+			}
+		}else if (op == 'I') {
+			read += length;
+			mismatch_length += length;
+			new_cigar = store_previous_m (0, &length_m, &length_x, &p, &s, new_cigar);			
+			new_cigar = add_cigar (new_cigar, &p, &s, length, 'I'); 
+		}else if (op == 'D') {
+			ref += length;
+			mismatch_length += length;
+			new_cigar = store_previous_m (0, &length_m, &length_x, &p, &s, new_cigar);			
+			new_cigar = add_cigar (new_cigar, &p, &s, length, 'D'); 
+		}
+	}
+	new_cigar = store_previous_m (0, &length_m, &length_x, &p, &s, new_cigar);
+	
+	length = readLen - read_end1 - 1;
+	if (length > 0) new_cigar = add_cigar(new_cigar, &p, &s, length, 'S');
+	
+	(*cigarLen) = p;	
+	free(*cigar);
+	(*cigar) = new_cigar;
+	return mismatch_length;
+}
 
-inline uint32_t cigar_int_to_len (uint32_t cigar_int)
-{
-	return cigar_int >> BAM_CIGAR_SHIFT;
-}*/
